function [w, efit, datamatrix, F] = lp_fitfun(X, F, opt)

% [w, efit, datamatrix, F] = lp_fitfun(X, F, opt)
%
% lp_fitfum executes LP to approximate the kernel signature
%
% X domain matrix structure, prepared by ldfm_dataprepare
% F frequency matrix, each column is frqs for all dimensions
% opt:
%     do_sparse - includes L1 norm on the output weights in minimization
%     fit_ewight - the mixing gamma parameter
%     do_taylor - find delta omega for continuos frequencies
%     delta_div - alternates the max delta (in X.d{i}.max_delta) 
%
% Output:
%   w    - weights
%   efit - L-inf error of the fit
%   datamatrix - sample values of the respective cosines, datamatrix * w' =
%                kernel signature
%   F    - updated set of frequencies (if do_taylor used)
%
% For more details see Chum: Low Dimensional Explicit Feature Maps, ICCV'15
% http://cmp.felk.cvut.cz/~chum
% (C) Ondrej Chum 2015

dopt.delta_div = 1;
dopt.fit_weight = 3;
dopt.do_sparse = 1;
dopt.do_taylor = 1;
dopt.free_constant = 0;
dopt.error_bound = inf;
dopt.datamatrix_fun = @ldfm_datamatrix;
dopt.datapardif_fun = @ldfm_datapardif;

if nargin < 3
    opt = dopt;
else
    opt = optmerge(dopt, opt);
end

if exist('cplexlp') == 6
    LP = @cplexlp;
else
    LP = @linprog;
end

[datamatrix, f0] = opt.datamatrix_fun(X,F);

nfrq = size(datamatrix, 2);
npts = size(datamatrix, 1);
ndim = size(X.domain,1);

fn = X.values;

% Summing the functions

b_des = [fn'; -fn'];
A_des = [datamatrix;-datamatrix];

% Taylor expansion
stcol = nfrq+1;
A_deriv = [];

for d = 1 : ndim
    if opt.do_taylor & isfield(X.d{d},'max_delta') & X.d{d}.max_delta > 0
        T.mdel = X.d{d}.max_delta / opt.delta_div;
        [dC, T.u] = opt.datapardif_fun(X,F,d);
        A_deriv = [A_deriv, dC];
        clear dC
        
        ufrq = length(T.u);
        T.stcol = stcol;
        T.ecol = stcol+ufrq-1;
        stcol = stcol + ufrq;
    else
        T.mdel = -1;
        T.stcol = stcol;
        T.ecol = stcol-1;
    end
    Ts{d} = T;
end
        
dcols = stcol - nfrq - 1;
A = zeros(2*npts + 2*dcols, stcol);
A(1:2*npts,1:nfrq) = A_des;
clear A_des;
A(1:npts,nfrq+1:end-1) = A_deriv;
A(npts+1:2*npts,nfrq+1:end-1) = -A_deriv;
clear A_deriv;

slrow = 2*npts;
for d = 1 : ndim
    T = Ts{d};
    if T.stcol <= T.ecol
        ufrq = length(T.u);
        for i = 1:ufrq
           fq = T.u(i);
           A(slrow+ 2*i-1,[fq, T.stcol+i-1]) = [-T.mdel, 1];
           A(slrow+ 2*i,[fq, T.stcol+i-1]) = [-T.mdel, -1];
        end
        slrow = slrow + 2*ufrq;
    end
end


eps_idx = size(A,2);

if isfield(X,'eps_multiplier')
    epsmult = X.eps_multiplier;
else
    epsmult = ones(size(fn));
end
A(1:2*npts,eps_idx) = [-epsmult(:);-epsmult(:)];

b = [b_des; zeros(2*dcols,1)];

LB = [zeros(nfrq,1);-inf(dcols,1); 0];
UB = inf(nfrq+1+dcols,1);

f = zeros(size(A,2),1);
if opt.do_sparse
    f(1:nfrq) = f0;
    if isinf(opt.error_bound)
      f(eps_idx) = opt.fit_weight;
    else
      UB(eps_idx) = opt.error_bound;
    end
    if opt.free_constant
        constidx = find(f0 == 1);
        f(constidx) = 0;
    end
else
    f(eps_idx) = 1;
end

Ae = [];
be = [];

[ws, fval, eflag] = LP(f, A, b, Ae, be, LB, UB);

if isempty(ws)
    w = [];
    efit = inf;
    return;
end

w = ws(1:nfrq);
efit = ws(eps_idx);

for d = 1 : ndim
    T = Ts{d};
    if T.stcol <= T.ecol
        wp = ws(T.stcol:T.ecol);
        del = wp ./ w(T.u);
        del(del < -T.mdel) = -T.mdel;
        del(del > T.mdel) = T.mdel;
        del(isnan(del)) = 0;
        F(d,T.u) = F(d,T.u) + del';
    end
end        
